@@ -1,7 +1,7 @@
 /*
  * event.c	Server event handling
  *
- * Version:	$Id: event.c,v 1.82 2008/01/09 15:59:26 aland Exp $
+ * Version:	$Id: event.c,v 1.87 2008/02/12 10:08:32 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: event.c,v 1.82 2008/01/09 15:59:26 aland Exp $")
+RCSID("$Id: event.c,v 1.87 2008/02/12 10:08:32 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -88,6 +88,7 @@ static rad_listen_t	*proxy_listeners[32];
 static void request_post_handler(REQUEST *request);
 static void wait_a_bit(void *ctx);
 static void event_poll_fd(void *ctx);
+static void event_socket_handler(fr_event_list_t *xel, UNUSED int fd, void *ctx);
 
 static void NEVER_RETURNS _rad_panic(const char *file, unsigned int line,
 				    const char *msg)
@@ -298,18 +299,9 @@ static int insert_into_proxy_hash(REQUEST *request)
 		rad_listen_t *proxy_listener;
 
 		/*
-		 *	Allocate a new proxy fd.  This function adds it
-		 *	into the list of listeners.
-		 *
-		 *	FIXME!: Call event_fd_insert()!  Otherwise this
-		 *	FD will never be used in select()!
-		 *
-		 *	We probably want to add RADIUS_SIGNAL_SELF_FD,
-		 *	which tells us to walk over the listeners,
-		 *	adding a new FD.
-		 *
-		 *	Hmm... maybe do this for the detail file, too.
-		 *	that will simplify some of the rest of the code.
+		 *	Allocate a new proxy fd.  This function adds
+		 *	it to the tail of the list of listeners.  With
+		 *	some care, this can be thread-safe.
 		 */
 		proxy_listener = proxy_new_listener();
 		if (!proxy_listener) {
@@ -324,8 +316,6 @@ static int insert_into_proxy_hash(REQUEST *request)
 		found = -1;
 		proxy = proxy_listener->fd;
 		for (i = 0; i < 32; i++) {
-			DEBUG2("PROXY %d %d", i, proxy_fds[(proxy + i) & 0x1f]);
-
 			/*
 			 *	Found a free entry.  Save the socket,
 			 *	and remember where we saved it.
@@ -342,7 +332,7 @@ static int insert_into_proxy_hash(REQUEST *request)
 		if (!fr_packet_list_socket_add(proxy_list, proxy_listener->fd)) {
 			PTHREAD_MUTEX_UNLOCK(&proxy_mutex);
 			DEBUG2("ERROR: Failed to create a new socket for proxying requests.");
-			return 0; /* leak proxy_listener */
+			return 0;
 
 		}
 
@@ -351,6 +341,12 @@ static int insert_into_proxy_hash(REQUEST *request)
 			DEBUG2("ERROR: Failed to create a new socket for proxying requests.");
 			return 0;
 		}
+
+		/*
+		 *	Signal the main thread to add the new FD to the list
+		 *	of listening FD's.
+		 */
+		radius_signal_self(RADIUS_SIGNAL_SELF_NEW_FD);
 	}
 	rad_assert(request->proxy->sockfd >= 0);
 
@@ -1160,7 +1156,7 @@ static int successfully_proxied_request(REQUEST *request)
 	 *
 	 *	FIXME: This should really be a serious error.
 	 */
-	if (request->in_proxy_hash || request->proxy) {
+	if (request->in_proxy_hash) {
 		return 0;
 	}
 
@@ -1418,7 +1414,7 @@ static void request_post_handler(REQUEST *request)
 	}
 
 	if (request->root->proxy_requests &&
-	    !request->proxy &&
+	    !request->in_proxy_hash &&
 	    (request->reply->code == 0) &&
 	    (request->packet->dst_port != 0) &&
 	    (request->packet->code != PW_STATUS_SERVER)) {
@@ -2146,6 +2142,22 @@ static void handle_signal_self(int flag)
 
 		fr_event_loop_exit(el, 0x80);
 	}
+
+	if ((flag & RADIUS_SIGNAL_SELF_NEW_FD) != 0) {
+		rad_listen_t *this;
+		
+		for (this = mainconfig.listen;
+		     this != NULL;
+		     this = this->next) {
+			if (this->type != RAD_LISTEN_PROXY) continue;
+			
+			if (!fr_event_fd_insert(el, 0, this->fd,
+						event_socket_handler, this)) {
+				radlog(L_ERR, "Failed remembering handle for proxy socket!");
+				_exit(1);
+			}
+		}
+	}
 }
 
 #ifdef __MINGW32__
@@ -2226,7 +2238,7 @@ static void event_reset_fd(rad_listen_t *listener, int fd, int usec)
 
 	tv_add(&when, usec);	
 
-	if (!fr_event_fd_delete(el, 0, fd)) {
+	if ((fd >= 0) && !fr_event_fd_delete(el, 0, fd)) {
 		rad_panic("Failed deleting fd");
 	}
 	
@@ -2370,7 +2382,8 @@ static void event_poll_fd(void *ctx)
 	 *	Ignore the detail file if we're busy with other work.
 	 */
 	if ((listener->type == RAD_LISTEN_DETAIL) && !read_from_detail) {
-		goto reset;
+		event_reset_fd(listener, listener->fd, USEC);
+		return;
 	}
 
 	/*
@@ -2397,17 +2410,7 @@ static void event_poll_fd(void *ctx)
 	 *	closed.  Re-set the polling timer and return.
 	 */
 	if (listener->fd < 0) {
-		struct timeval when;
-
-	reset:
-		gettimeofday(&when, NULL);
-		when.tv_sec += 1;
-		
-		if (!fr_event_insert(el, event_poll_fd, listener,
-				       &when, NULL)) {
-			rad_panic("Failed inserting event");
-		}
-
+		event_reset_fd(listener, -1, USEC);
 		return;
 	}
 
@@ -2438,14 +2441,18 @@ static void event_status(struct timeval *wake)
 			just_started = FALSE;
 		}
 		return;
+
+	} else {
+		read_from_detail = TRUE;
 	}
 
 	if (!wake) {
 		DEBUG("Ready to process requests.");
+
 	} else if ((wake->tv_sec != 0) ||
 		   (wake->tv_usec >= 100000)) {
-		DEBUG("Waking up in %d.%01d seconds.\n",
-		      (int) wake->tv_sec, (int) wake->tv_usec / 100000);
+		DEBUG("Waking up in %d.%01u seconds.\n",
+		      (int) wake->tv_sec, (unsigned int) wake->tv_usec / 100000);
 	}
 }
 
@@ -2567,6 +2574,10 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 			has_detail_listener = TRUE;
 			break;
 
+		case RAD_LISTEN_SNMP:
+			DEBUG("Listening on SNMP %s", buffer);
+			break;
+
 		case RAD_LISTEN_PROXY:
 			rad_assert(proxy_fds[this->fd & 0x1f] == -1);
 			rad_assert(proxy_listeners[this->fd & 0x1f] == NULL);
@@ -2577,11 +2588,7 @@ int radius_event_init(CONF_SECTION *cs, int spawn_flag)
 							 this->fd)) {
 				rad_assert(0 == 1);
 			}
-			break;
-
-		case RAD_LISTEN_SNMP:
-			DEBUG("Listening on SNMP %s", buffer);
-			break;
+			/* FALL-THROUGH */
 
 		default:
 			DEBUG("Listening on %s", buffer);
