@@ -1,7 +1,7 @@
 /*
  * modcall.c
  *
- * Version:	$Id: modcall.c,v 1.95 2008/01/05 01:06:51 nbk Exp $
+ * Version:	$Id: modcall.c,v 1.96 2008/02/10 11:30:05 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: modcall.c,v 1.95 2008/01/05 01:06:51 nbk Exp $")
+RCSID("$Id: modcall.c,v 1.96 2008/02/10 11:30:05 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modpriv.h>
@@ -1617,8 +1617,13 @@ static modcallable *do_compile_modsingle(modcallable *parent,
 	csingle = mod_singletocallable(single);
 	csingle->parent = parent;
 	csingle->next = NULL;
-	memcpy(csingle->actions, defaultactions[component][grouptype],
-	       sizeof csingle->actions);
+	if (!parent || (component != RLM_COMPONENT_AUTH)) {
+		memcpy(csingle->actions, defaultactions[component][grouptype],
+		       sizeof csingle->actions);
+	} else { /* inside Auth-Type has different rules */
+		memcpy(csingle->actions, defaultactions[RLM_COMPONENT_AUTZ][grouptype],
+		       sizeof csingle->actions);
+	}
 	rad_assert(modrefname != NULL);
 	csingle->name = modrefname;
 	csingle->type = MOD_SINGLE;
@@ -1794,7 +1799,11 @@ static modcallable *do_compile_modgroup(modcallable *parent,
 	 */
 	for (i = 0; i < RLM_MODULE_NUMCODES; i++) {
 		if (!c->actions[i]) {
-			c->actions[i] = defaultactions[component][parentgrouptype][i];
+			if (!parent || (component != RLM_COMPONENT_AUTH)) {
+				c->actions[i] = defaultactions[component][parentgrouptype][i];
+			} else { /* inside Auth-Type has different rules */
+				c->actions[i] = defaultactions[RLM_COMPONENT_AUTZ][parentgrouptype][i];
+			}
 		}
 	}
 
